package function

import (
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"gitlab.smartblocklab.com/smartcontracts/common/model/offerCC"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/offersessionCC/mngr"
	"gitlab.smartblocklab.com/smartcontracts/common/model/transactionCC"
)

func AddOffer(userParameter *requests.Parameter, systemParameter *requests.Parameter, manager chaincode.IManager) (interface{}, error) {
	newOfferRequest, err := offerCC.Unmarshal_OfferRequest(userParameter.Param)
	if err != nil {
		return nil, err
	}

	operationID, err := requests.Unmarshal(systemParameter.Param)
	if err != nil {
		return nil, err
	}

	if newOfferRequest == nil {
		return nil, errors.Errorf("Out user can't be null")
	}

	if len(operationID.Id) == 0 {
		return nil, errors.Errorf("OperationID can't be empty")
	}

	if len(newOfferRequest.OfferRecordStatus) == 0 {
		newOfferRequest.OfferRecordStatus = offerCC.OfferRecordStatusRequest
	}

	err = newOfferRequest.Validate()
	if err != nil {
		return nil, err
	}

	outOfferRecord := &offerCC.OfferRecord{
		OperationID:       operationID.Id,
		UserAccountInfo:   newOfferRequest.OutUserInfo.UserAccountInfo,
		Amount:            newOfferRequest.OutUserInfo.Amount,
		InOutType:         transactionCC.OUTPUT,
		Timestamp:         transactionCC.Timestamp(manager.GetTime()),
		OfferRecordStatus: newOfferRequest.OfferRecordStatus,
		ReplayedKey:       newOfferRequest.ReplayedKey,
	}

	inOfferRecord := &offerCC.OfferRecord{
		OperationID:       operationID.Id,
		UserAccountInfo:   newOfferRequest.InUserInfo.UserAccountInfo,
		Amount:            newOfferRequest.InUserInfo.Amount,
		InOutType:         transactionCC.INPUT,
		Timestamp:         transactionCC.Timestamp(manager.GetTime()),
		OfferRecordStatus: newOfferRequest.OfferRecordStatus,
		ReplayedKey:       newOfferRequest.ReplayedKey,
	}

	if outOfferRecord.UserAccountInfo == nil {
		return nil, errors.Errorf("Out user can't by null or empty")
	}

	if outOfferRecord.UserAccountInfo.UserID != userParameter.UserID {
		return nil, errors.Errorf("Can't %s because signer UserID %s not equals with out UserID %s", invoke.AddOfferFunction, userParameter.UserID, outOfferRecord.UserAccountInfo.UserID)
	}

	return mngr.GetOfferSessionManager(manager).AddOffer(outOfferRecord, inOfferRecord, operationID)
}
