package mngr

import (
	"gitlab.smartblocklab.com/smartcontracts/common/model/offerCC"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"errors"
	"gitlab.smartblocklab.com/smartcontracts/common/model/transactionCC"
	"gitlab.smartblocklab.com/smartcontracts/common/model/keyValueCC"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
)

func (this OfferSessionManagerImpl) GetOffers(userID *requests.Id) (*offerCC.Offers, error) {
	result := &offerCC.Offers{
		Offers: make(map[string]*offerCC.Offer, 0),
	}

	//return []string{this.UserAccountInfo.UserID, this.UserAccountInfo.WalletID, this.UserAccountInfo.AccountID, this.Amount.CurrencyType, this.InOutType.String(), this.OperationID, this.OfferRecordStatus.String(), manager.GetTxID()}
	validObjects, err := this.GetObjectListByKeysWithFilter(&offerCC.OfferRecord{}, []string{userID.Id})
	if err != nil {
		return nil, err
	}

	for _, voOfferRequest := range validObjects {
		offerRecord, err := offerCC.Cast(voOfferRequest)
		if err != nil {
			return nil, err
		}
		if offerRecord == nil {
			return nil, errors.New("offerRecord is nil")
		}
		offer := result.Offers[offerRecord.OperationID]
		if offer == nil {
			offer = &offerCC.Offer{}
			offer.Properties = make([]*keyValueCC.KeyValue, 0)
			offer.OfferStatus = make([]*offerCC.OfferStatus, 0)
			offer.Replay = make([]*offerCC.OfferRecordPair, 0)
			offer.OperationId = offerRecord.OperationID
		}

		offerRecordByDependencyKey, err := this.FindOfferRecordByDependencyKey(offerRecord.DependencyKey)
		if err != nil {
			return nil, err
		}

		offerRecordPair := &offerCC.OfferRecordPair{}
		offerRecordPair.InOutType = offerRecord.InOutType

		if offerRecord.InOutType == transactionCC.INPUT {
			offerRecordPair.In = offerRecord
			offerRecordPair.Out = offerRecordByDependencyKey
		} else if offerRecord.InOutType == transactionCC.OUTPUT {
			offerRecordPair.Out = offerRecord
			offerRecordPair.In = offerRecordByDependencyKey
		}

		if len(offerRecord.ReplayedKey) != 0 {
			offer.Replay = append(offer.Replay, offerRecordPair)
		} else {
			offer.Request = offerRecordPair
		}

		result.Offers[offerRecord.OperationID] = offer
	}

	operationIdArray := make([]requests.Id, 0, len(result.Offers))
	for operationId := range result.Offers {
		operationIdArray = append(operationIdArray, requests.Id{operationId})
	}

	keyArrayId := &requests.ArrayId{ArrayId: operationIdArray}
	statusKeyValueArray, err := this.OfferStatusCC.GetKeyValueArray(keyArrayId)
	if err != nil {
		return nil, err
	}
	if statusKeyValueArray != nil && statusKeyValueArray.KeyValue != nil && len(statusKeyValueArray.KeyValue) > 0 {
		for _, statusKeyValue := range statusKeyValueArray.KeyValue {
			offer := result.Offers[statusKeyValue.Keys[0]]
			offerStatus := offerCC.OfferStatus(statusKeyValue.Value)
			offer.OfferStatus = append(offer.OfferStatus, &offerStatus)
			result.Offers[statusKeyValue.Keys[0]] = offer
		}
	}

	propertiesKeyValueArray, err := this.OfferPropertiesCC.GetKeyValueArray(keyArrayId)
	if err != nil {
		return nil, err
	}
	if propertiesKeyValueArray != nil && propertiesKeyValueArray.KeyValue != nil && len(propertiesKeyValueArray.KeyValue) > 0 {
		for _, propertiesKeyValue := range propertiesKeyValueArray.KeyValue {
			offer := result.Offers[propertiesKeyValue.Keys[0]]
			offer.Properties = append(offer.Properties, &propertiesKeyValue)
			result.Offers[propertiesKeyValue.Keys[0]] = offer
		}
	}

	return result, nil
}


func (this *OfferSessionManagerImpl) FindOfferRecordByDependencyKey(key string) (*offerCC.OfferRecord, error) {
	chaincode.GlobalLogger.Infof("FindOfferRecordByDependencyKey(key: %s)", key)

	if len(key) == 0 {
		return nil, chaincode.GlobalLogger.Errorf("an empty Key, cannot process, FindOfferRecordByDependencyKey request")
	}

	validObject, err := this.GetObjectByCompositeKey(&offerCC.OfferRecord{}, key)
	if err != nil {
		return nil, err
	}

	if validObject == nil {
		return nil, chaincode.GlobalLogger.Errorf("OfferRecord not found by key: %s", key)
	}

	return offerCC.Cast(validObject)
}