package mngr

import (
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/iaccount"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/ioffer"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/itransaction"
	"gitlab.smartblocklab.com/smartcontracts/common/remote/remote_account"
	"gitlab.smartblocklab.com/smartcontracts/common/remote/remote_transaction"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/ikeyvalue"
	"gitlab.smartblocklab.com/smartcontracts/common/remote/remote_keyvalue"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/ioffersession"
)

type OfferSessionManagerImpl struct {
	chaincode.IManager
	AccountCC     iaccount.IAccountManager
	TransactionCC itransaction.ITransactionManager
	OfferStatusCC ikeyvalue.IKeyValueManager
	OfferPropertiesCC ikeyvalue.IKeyValueManager
}

func GetOfferSessionManager(manager chaincode.IManager) ioffersession.IOfferSessionManager {
	accountRemoteManager := remote_account.GetAccountRemoteManager(manager, invoke.AccountChannel, invoke.AccountCC)
	transactionRemoteManager := remote_transaction.GetTransactionRemoteManager(manager, invoke.TransactionChannel, invoke.TransactionCC)
	offerStatusRemoteManager := remote_keyvalue.GetKeyValueRemoteManager(manager, invoke.OfferStatusChannel, invoke.OfferStatusCC)
	offerPropertiesRemoteManager := remote_keyvalue.GetKeyValueRemoteManager(manager, invoke.OfferPropertiesChannel, invoke.OfferPropertiesCC)

	return &OfferSessionManagerImpl{
		manager,
		accountRemoteManager,
		transactionRemoteManager,
		offerStatusRemoteManager,
		offerPropertiesRemoteManager,
	}
}

func GetOfferSessionManagerMock(manager chaincode.IManager, accountRemoteManager iaccount.IAccountManager, transactionRemoteManager itransaction.ITransactionManager, offerStatusRemoteManager ikeyvalue.IKeyValueManager, offerPropertiesRemoteManager ikeyvalue.IKeyValueManager) ioffer.IOfferManager {
	return &OfferSessionManagerImpl{
		manager,
		accountRemoteManager,
		transactionRemoteManager,
		offerStatusRemoteManager,
		offerPropertiesRemoteManager,
	}
}
