package accountCC

import (
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"encoding/json"
	"reflect"
)

type Wallet struct {
	WalletID string           `json:"wallet_id,omitempty"`
	UserID   string           `json:"user_id,omitempty"`
	Type     AccountCCType    `json:"type,omitempty"`
	Status   AccountStatus    `json:"status,omitempty"`
}

func (this Wallet) GetObjectType() string  {
	return "Wallet";
}

func (this Wallet) CheckBeforeAdd(manager chaincode.IManager) error {
	// TODO check user exists
	// TODO when add check AccountStatus_ACTIVE : wallet.Status = model.AccountStatus_ACTIVE

	return nil
}

func (this Wallet) CheckBeforeUpdate(updatedObject chaincode.ValidObject, manager chaincode.IManager) error {
	// TODO check user exists
	// TODO when add check AccountStatus_ACTIVE : wallet.Status = model.AccountStatus_ACTIVE
	updatedWallet, err := Cast_Wallet(updatedObject)
	if err != nil {
		return err
	}

	if updatedWallet.UserID != this.UserID {
		return errors.Errorf("Wallet associated with wrong UserID, expecting %s, while received %s", updatedWallet.UserID, this.UserID)
	}

	return nil
}

func (this Wallet) New() chaincode.ValidObject {
	return &Wallet{}
}

func (this Wallet) GetKey(manager chaincode.IManager) []string {
	return []string{this.UserID, this.WalletID}
}

func (this Wallet) Validate() error {
	if len(this.WalletID) == 0 {
		return errors.New("WalletID is empty")
	}
	if len(this.UserID) == 0 {
		return errors.New("UserID is empty")
	}

	if &this.Type == nil || len(string(this.Type)) == 0 {
		return errors.New("Type is empty")
	}

	if this.Type != Main && this.Type != Additional {
		return errors.New("Type need to be Main or Additional")
	}

	if &this.Status == nil || len(string(this.Status)) == 0 {
		return errors.New("Status is empty")
	}

	return nil
}

func Unmarshal_Wallet(data []byte) (*Wallet, error) {
	if data == nil {
		return nil, chaincode.GlobalLogger.Error("can't Unmarshal nil data")
	}

	var obj Wallet
	err := json.Unmarshal(data, &obj)
	return &obj, err;
}

func Marshal_Wallet(data *Wallet) ([]byte, error) {
	return json.Marshal(data);
}

func Cast_Wallet(validObject chaincode.ValidObject) (*Wallet, error) {
	if validObject == nil {
		return nil, nil
	}

	if reflect.ValueOf(validObject).Kind() != reflect.Ptr {
		return nil, errors.New("check validObjectc is not reference")
	}

	casted, ok := validObject.(*Wallet)
	if !ok {
		return nil, errors.New("Problem to get object")
	}

	return casted, nil;
}

