package accountCC

import (
	"github.com/hyperledger/fabric/core/chaincode/shim"
	"encoding/json"
	"gitlab.smartblocklab.com/smartcontracts/common/logging"
	"github.com/hyperledger/fabric/protos/peer"
	"fmt"
	"errors"
)

// invoke_GetAccountInfoByAccountID
type AccountStatus string

const (
	AccountStatus_BLOCKED AccountStatus = "BLOCKED"
	AccountStatus_ACTIVE  AccountStatus = "ACTIVE"
)

type AccountInfo struct {
	UserID    string        `json:"user_id,omitempty"`
	WalletID  string        `json:"wallet_id,omitempty"`
	AccountID string        `json:"account_id,omitempty"`
	Status    AccountStatus `json:"status,omitempty"`
}

func (s AccountStatus) String() string {
	return string(s)
}

func (obj *AccountInfo) Response() peer.Response {
	return shim.Success(obj.Bytes())
}

func (obj *AccountInfo) Bytes() []byte {
	if b, err := json.Marshal(*obj); err != nil {
		logging.Get().Errorf("failed to marshal MarketBalance, failed due to %s", err.Error())
		return []byte{}
	} else {
		return b
	}
}

func (accountInfo AccountInfo) Marshal() peer.Response {
	bytes, err := json.Marshal(accountInfo)
	if err != nil {
		errMsg := fmt.Sprintf("Failed to serialize accountInfo. Error: %s", err)
		return shim.Error(errMsg)
	}

	return shim.Success(bytes)
}

func (this *AccountInfo) Validate() error {
	if this == nil {
		return errors.New("AccountInfo can't be nil")
	}

	if len(this.AccountID) == 0 {
		return errors.New("AccountID can't be nil")
	}

	if len(this.WalletID) == 0 {
		return errors.New("WalletID can't be nil")
	}

	if len(this.UserID) == 0 {
		return errors.New("UserID can't be nil")
	}

	if len(this.Status) == 0 {
		return errors.New("Status can't be nil")
	}

	return nil
}