package cryptocurrencyCC

import (
	"gitlab.smartblocklab.com/smartcontracts/common/nums"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
)

// Defines operations supported by chaincode
const (
	LAST_IMPORT_TIME = "LAST_IMPORT_TIME"
	LAST_IMPORT_KEYS = "LAST_IMPORT_KEYS"
)

const (
	WAITING_SECONDS = 5 * 60
)

type CryptocurrencyInput struct {
	Time                  int64                    `json:"time"`
	ExternalTransactionID string                   `json:"external_transaction_id"`
	OutExternalAccountID  string                   `json:"out_external_account_id"`
	InExternalAccountID   string                   `json:"in_external_account_id"`
	Amount                nums.PlainCurrencyNumber `json:"amount"`
	Category              string                   `json:"category"`
	Confirmations         string                   `json:"confirmations"`
}

func (cryptocurrencyInput CryptocurrencyInput) ToString() {
	chaincode.GlobalLogger.Infof("CryptocurrencyInput{Time: %d, CurrencyType: %s, ExternalTransactionID: %s, OutExternalAccountID: %s, InExternalAccountID: %s, Amount: %d}",
		cryptocurrencyInput.Time,
		cryptocurrencyInput.Amount.CurrencyType,
		cryptocurrencyInput.ExternalTransactionID,
		cryptocurrencyInput.OutExternalAccountID,
		cryptocurrencyInput.InExternalAccountID,
		cryptocurrencyInput.Amount)
}

type CryptocurrencyListInput struct {
	Cryptocurrencies []CryptocurrencyInput `json:"cryptocurrencies"`
}

type resultStatus string

const (
	Ok    resultStatus = "OK"
	Error resultStatus = "ERROR"
)

type CryptocurrencyResult struct {
	ExternalTransactionID string       `json:"external_transaction_id"`
	Status                resultStatus `json:"status"`
	CommonMessage         string       `json:"common_message"`
	DetailMessage         string       `json:"detail_message"`
}

type ImportCryptocurrencyResult struct {
	CryptocurrencyResultList []CryptocurrencyResult `json:"add_cryptocurrency_result_list"`
}

type DaemonNameKeys map[string]map[string]bool

type LastImportKeys struct {
	MapDaemonNameKeys DaemonNameKeys `json:"keys"`
}

type AddCryptocurrenciesArg struct {
	CryptocurrencyListInput CryptocurrencyListInput `json:"cryptocurrency_list_input"`
}
