package keyValueCC

import (
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"encoding/json"
	"reflect"
)

type KeyValue struct {
	Keys  []string `json:"keys,omitempty"`
	Value string `json:"value,omitempty"`
}

func (this KeyValue) GetObjectType() string  {
	return "KeyValue";
}

type KeyValueArray struct {
	KeyValue []KeyValue `json:"key_value,omitempty"`
}

func UnmarshalKeyValueArray(data []byte) (*KeyValueArray, error) {
	if data == nil {
		return nil, chaincode.GlobalLogger.Error("can't Unmarshal nil data")
	}

	var obj KeyValueArray
	err := json.Unmarshal(data, &obj)
	return &obj, err;
}

func (this KeyValue) CheckBeforeAdd(manager chaincode.IManager) error {
	return nil
}

func (this KeyValue) CheckBeforeUpdate(updatedObject chaincode.ValidObject, manager chaincode.IManager) error {
	return nil
}

func (this KeyValue) New() chaincode.ValidObject {
	return &KeyValue{}
}

func (this KeyValue) GetKey(manager chaincode.IManager) []string {
	return append(this.Keys, this.Value)
}

func (this KeyValue) Validate() error {
	errorMsg := ""

	if len(this.Keys) == 0 {
		errorMsg += "Keys is empty; "
	}

	return nil
}

func Unmarshal(data []byte) (*KeyValue, error) {
	if data == nil {
		return nil, chaincode.GlobalLogger.Error("can't Unmarshal nil data")
	}

	var obj KeyValue
	err := json.Unmarshal(data, &obj)
	return &obj, err;
}

func Marshal(data *KeyValue) ([]byte, error) {
	return json.Marshal(data);
}

func Cast(validObject chaincode.ValidObject) (*KeyValue, error) {
	if validObject == nil {
		return nil, nil
	}

	if reflect.ValueOf(validObject).Kind() != reflect.Ptr {
		return nil, errors.New("check validObjectc is not reference")
	}

	casted, ok := validObject.(*KeyValue)
	if !ok {
		return nil, errors.New("Problem to get object")
	}

	return casted, nil;
}
