package offerCC

import (
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"encoding/json"
	"gitlab.smartblocklab.com/smartcontracts/common/nums"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/model/accountCC"
	"gitlab.smartblocklab.com/smartcontracts/common/model/keyValueCC"
	"gitlab.smartblocklab.com/smartcontracts/common/model/transactionCC"
)

// OfferRequestInfo capture offer information relevant to request
type OfferRequestInfo struct {
	*accountCC.UserAccountInfo
	Amount nums.PlainCurrencyNumber `json:"amount,omitempty"`
}

func (offerRequestInfo OfferRequestInfo) Validate() error {
	if offerRequestInfo.UserAccountInfo == nil {
		return errors.New("UserAccountInfo is empty")
	}

	err := offerRequestInfo.UserAccountInfo.Validate()
	if err != nil {
		return err
	}

	err = offerRequestInfo.Amount.Validate()
	if err != nil {
		return err
	}
	return nil
}

type OfferRecords struct {
	OfferRecords []*OfferRecord `json:"offer_records,omitempty"`
}

type Offers struct {
	Offers map[string]*Offer `json:"offers,omitempty"`
}

type Offer struct {
	OperationId string                 `json:"operation_id,omitempty"`
	Request     *OfferRecordPair       `json:"request,omitempty"`
	Replay      []*OfferRecordPair     `json:"replay,omitempty"`
	OfferStatus []*OfferStatus         `json:"offer_status,omitempty"`
	Properties  []*keyValueCC.KeyValue `json:"properties,omitempty"`
}

// OfferWithStatusList keeps list of offers
type OfferRecordPair struct {
	Out       *OfferRecord                  `json:"out,omitempty"`
	In        *OfferRecord                  `json:"in,omitempty"`
	InOutType transactionCC.InputOutputType `json:"input_output_type"`
}

func Unmarshal_Offers(data []byte) (*Offers, error) {
	if data == nil {
		return nil, chaincode.GlobalLogger.Error("can't Unmarshal nil data")
	}

	var obj Offers
	err := json.Unmarshal(data, &obj)
	return &obj, err;
}

type OfferStatus string

const (
	OfferPending   OfferStatus = "OfferPending"
	OfferExecuting OfferStatus = "OfferExecuting"
	OfferAccepted  OfferStatus = "OfferAccepted"
	OfferRejected  OfferStatus = "OfferRejected"
	OfferCanceled  OfferStatus = "OfferCanceled"
	OfferCompleted OfferStatus = "OfferCompleted"
)

func (offerRequestStatus OfferStatus) String() string {
	return string(offerRequestStatus)
}

type OfferRecordStatus string

const (
	OfferRecordStatusRequest OfferRecordStatus = "OfferRecordStatusRequest"
	OfferRecordStatusReplay  OfferRecordStatus = "OfferRecordStatusReplay"
)

func (offerRecordStatus OfferRecordStatus) String() string {
	return string(offerRecordStatus)
}

type GetUserIncomingRequestsByStatusArg struct {
	UserID             string `json:"user_id"`
	OfferRequestStatus string `json:"offer_request_status"`
}

type GetRequestsByStatusArg struct {
	OfferRequestStatus string `json:"offer_request_status"`
}

type GetUserOutgoingRequestsByStatusArg struct {
	UserID             string `json:"user_id"`
	OfferRequestStatus string `json:"offer_request_status"`
}

type GetHistoryUserOutgoingRequestsArg struct {
	UserID string `json:"user_id"`
}

type GetHistoryUserIncomingRequestsArg struct {
	UserID string `json:"user_id"`
}

type SetOfferStatusByOperationIdArg struct {
	OperationID        string      `json:"operation_id"`
	OfferRequestStatus OfferStatus `json:"offer_request_status"`
}

type PutOfferPropertiesByOperationIDArg struct {
	OperationID string `json:"operation_id"`
	Key         string `json:"key,omitempty"`
	Value       string `json:"value,omitempty"`
}

type RequestsByStatusAndCurrencyType struct {
	OfferRequestStatus OfferStatus            `json:"offer_request_status,omitempty"`
	OutUserInfo        RequestsByCurrencyType `json:"out_info,omitempty"`
}

type PlainCurrencyNumberVar struct {
	CurrencyType string `json:"currency_type"`
}

type RequestsByCurrencyType struct {
	Amount PlainCurrencyNumberVar `json:"amount,omitempty"`
}

type GetRequestsByStatusAndCurrencyTypeArg struct {
	OfferRequestStatus string `json:"offer_request_status"`
	CurrencyType       string `json:"currency_type"`
}
